// *******************************************************************
//
// Copyright (c) 2013, Antmicro Ltd
//
// Authors:
// * Mateusz Holenko (mholenko@antmicro.com)
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// *******************************************************************
using System.Text;
using System.Linq;

namespace AntShell.Commands
{
    public interface ICommandDescription
    {
        string Name { get; }

        string[] AlternativeNames { get; }

        string Description { get; }

        void PrintHelp(ICommandInteraction writer);
    }

    public static class ICommandDescriptionHelper
    {
        public static string GetHelp(this ICommandDescription cd)
        {
            var helper = new StringBuilder(cd.Name);
            if(cd.AlternativeNames.Any())
            {
                helper.AppendFormat(" [ {0} ]", cd.AlternativeNames.Aggregate((x, y) => x + ", " + y));
            }
            helper.AppendFormat("\r\n{0}", cd.Description);
            return helper.ToString();
        }
    }
}

